import telnetlib
import os
import string

stream_ip = "127.0.0.1"
stream_port = 8080 #Port used for streaming
telnet_port = 4212 #Port used for telnet remote control
telnet_password = "v" #Password for VLC Streaming Server telnet control
vlc_path = "/Applications/VLC.app/Contents/MacOS/VLC" #VLC path, no whitespaces
recording_path = "/Users/Wizard/"
config_path = "channels.conf" #Where is config file? No whitespaces allowed
dvb_mode = "DVBC" #DVBT/DVBC
system_type = "UNIX" #WIN/UNIX

recording_status = "false"
now_playing = "false"
channels = []

class TelnetController:
				
	def __init__(self, prompt):			
		self.prompt = prompt
		self.tn = None		
		
	def login(self):
		global stream_ip
		global telnet_port
		global telnet_password
		
		self.tn = telnetlib.Telnet(stream_ip, telnet_port)		
		
		if telnet_password:
			self.tn.read_until('assword: ')
			self.tn.write(telnet_password + '\n')
		self.tn.read_until(self.prompt)
 
	def run_command(self, command):
		self.tn.write(command + '\n')
		response = self.tn.read_until(self.prompt)
		return self.__strip_output(command, response)
		
	
	def logout(self):
		self.tn.close()
		
		
	def run_atomic_command(self, command):
		self.login()
		command_output = self.run_command(command)
		self.logout()
		return command_output

		
	def __strip_output(self, command, response):
		lines = response.splitlines()
		# if our command was echoed back, remove it from the output
		if command in lines[0]:
			lines.pop(0)
		# remove the last element, which is the prompt being displayed again
		lines.pop()
		# append a newline to each line of output
		lines = [item + '\n' for item in lines]
		# join the list back into a string and return it
		return ''.join(lines)


def init_dvb(telnet, channels):
	telnet.run_command("del all")
	global stream_ip
	global stream_port
	
	print "Transmitting tuning parameters to VLC stream server\nSelected DVB mode: " + dvb_mode + "\n"
	
	for line in channels:	
		dst = stream_ip + ':' + str(stream_port)	
		tx_line = line		
		
		if dvb_mode == "DVBT":
			
			channel_name = tx_line[0].replace(' ','')
			dvb_frequency = tx_line[1]
			dvb_transmission = tx_line[7].lstrip("TRANSMISSION_MODE").rstrip("K")
			dvb_guard = tx_line[8].lstrip("GUARD_INTERVAL_1_")
			dvb_bandwidth = tx_line[3].lstrip("BANDWIDTH_").rstrip("_MHZ")			
			program_id = tx_line[12]						
			
			tx_buf = 'new ' + str(channel_name) + ' broadcast enabled' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' input ''dvb://''' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-adapter=0' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-frequency=' + str(dvb_frequency) + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-transmission='+ str(dvb_transmission) + ' \n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-guard='+ str(dvb_guard) + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-hierarchy=0' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-bandwidth='+ str(dvb_bandwidth) + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option program=' + str(program_id) + '' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option standard-mux=ps' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' output #standard{access=http,mux=ps,dst='+dst+'}' + '\n'		
			
		if dvb_mode == "DVBC":
			
			channel_name = tx_line[0].replace(' ','')
			dvb_frequency = tx_line[1]
			dvb_modulation = tx_line[5].lstrip("QAM_")
			program_id = tx_line[8]						
			
			tx_buf = 'new ' + str(channel_name) + ' broadcast enabled' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' input ''dvb://''' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-adapter=0' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-frequency=' + str(dvb_frequency) + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option program=' + str(program_id) + '' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-modulation=' + str(dvb_modulation) + '' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option standard-mux=ps' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' option dvb-srate=6900000' + '\n'
			tx_buf = tx_buf + 'setup ' + str(channel_name) + ' output #standard{access=http,mux=ps,dst='+dst+'}' + '\n'			
			
		print tx_buf
		
		telnet.run_command(tx_buf)
		tx_buf = ""
		
def clear():
	if system_type == "WIN":
		os.system("cls")
	else:
		os.system("clear")
	
def tune(telnet, channel_id):
	global now_playing
	global channels
		
	ch = iter(channels)
	i = 0
	while i <= int(int(channel_id)-1):
		channel = ch.next()		
		i = i + 1	
	clear()
	telnet.run_command("control " + str(now_playing) + " stop")
	os.system("killall VLC >/dev/null 2>&1 &")
	telnet.run_command("control " + str(channel[0].replace(' ','')) + " play")
	now_playing = str(channel[0].replace(' ',''))
	
def run(program, *args, **kw):
    # find executable
    mode = kw.get("mode", os.P_WAIT)
    for path in string.split(os.environ["PATH"], os.pathsep):
        file = os.path.join(path, program)
        try:
            return os.spawnv(mode, file, (file,) + args)
        except os.error:
            pass
    raise os.error, "cannot find executable"


		
telnet = TelnetController(">")
telnet.login()

f = open(config_path)
global channels

try:
	for line in f:
		line_ = line[:-1]
		data = line_.split(':') 	   
		channels.append (data)
		
finally:
	f.close()

init_dvb(telnet, channels)
running = "true"

while running == "true":
	print " -- VLC DVB STREAMING -- \n\n"
	j = 1
	for line in channels:
		print '['+str(j)+'] ' + line[0].replace(' ','')
		j = j + 1
	print "\n"
	
	print "Streaming: ["+now_playing+"]"
	
	if recording_status == "true":
		print "RECORDING: [" + now_playing+"]"
	channel = raw_input("Tune channel ([q]uits, [rec]ords, [tel]kku.com) > ")
	
	if not channel == "q":
		if not channel == "rec" and not channel == "tel":
			try:
				if int(channel) <= len(channels):
					tune(telnet, channel)		
					#run(vlc_path, "-vvv http://"+stream_ip+":8080", mode=os.P_NOWAIT)
					os.system("/Applications/VLC.app/Contents/MacOS/VLC -vvv http://"+stream_ip+":8080  >/dev/null 2>&1 &")
					recording_status = "false"
			except:
				None
				
		else:
			if channel == "rec":
				filename = raw_input("Recording filename ([c]ancel) > ")	
				
				if not filename == "c":
					dst = (recording_path + filename + ".mpg").replace(' ','')
					
					os.system("/Applications/VLC.app/Contents/MacOS/VLC -vvv http://"+stream_ip+":8080 --sout #duplicate{dst=std{access=file,mux=ps,dst=\""+dst+"\"}} &")
					print "Recording " + channel + " > " +dst+ ""
					recording_status = "true"
				clear()
			if channel == "tel":
				print "telkku"
				os.system("open http://www.telkku.com  >/dev/null 2>&1 &")
			
	else:
		running = "false"
		os.system("killall VLC >/dev/null 2>&1 &")
